@@ -430,7 +430,6 @@ int map_block2 (struct minix2_inode *inode, unsigned int blknr)
 	unsigned int dind[BLOCK_SIZE >> 2];
 	unsigned int tind[BLOCK_SIZE >> 2];
 	int blk_chg, block, result;
-	int ind_nr;
 
 	if (blknr < 7)
 		return check_zone_nr2 (inode->i_zone + blknr, &changed);
@@ -598,11 +597,11 @@ void read_tables(void)
 	}
 	get_dirsize ();
 	if (show) {
-		printf("%d inodes\n",INODES);
-		printf("%d blocks\n",ZONES);
-		printf("Firstdatazone=%d (%d)\n",FIRSTZONE,NORM_FIRSTZONE);
+		printf("%ld inodes\n",INODES);
+		printf("%ld blocks\n",ZONES);
+		printf("Firstdatazone=%ld (%ld)\n",FIRSTZONE,NORM_FIRSTZONE);
 		printf("Zonesize=%d\n",BLOCK_SIZE<<ZONESIZE);
-		printf("Maxsize=%d\n",MAXSIZE);
+		printf("Maxsize=%ld\n",MAXSIZE);
 		printf("Filesystem state=%d\n", Super.s_state);
 		printf("namelen=%d\n\n",namelen);
 	}
@@ -807,7 +806,7 @@ static void add_zone_ind(unsigned short * znr, int * corrected)
 		write_block(block, blk);
 }
 
-static void add_zone_ind2 (unsigned long *znr, int *corrected)
+static void add_zone_ind2 (unsigned int *znr, int *corrected)
 {
 	static char blk[BLOCK_SIZE];
 	int i, chg_blk = 0;
@@ -818,7 +817,7 @@ static void add_zone_ind2 (unsigned long *znr, int *corrected)
 		return;
 	read_block (block, blk);
 	for (i = 0; i < BLOCK_SIZE >> 2; i++)
-		add_zone2 (i + (unsigned long *) blk, &chg_blk);
+		add_zone2 (i + (unsigned int *) blk, &chg_blk);
 	if (chg_blk)
 		write_block (block, blk);
 }
@@ -840,7 +839,7 @@ static void add_zone_dind(unsigned short * znr, int * corrected)
 }
 
 static void
-add_zone_dind2 (unsigned long *znr, int *corrected)
+add_zone_dind2 (unsigned int *znr, int *corrected)
 {
 	static char blk[BLOCK_SIZE];
 	int i, blk_chg = 0;
@@ -851,13 +850,13 @@ add_zone_dind2 (unsigned long *znr, int *corrected)
 		return;
 	read_block (block, blk);
 	for (i = 0; i < BLOCK_SIZE >> 2; i++)
-		add_zone_ind2 (i + (unsigned long *) blk, &blk_chg);
+		add_zone_ind2 (i + (unsigned int *) blk, &blk_chg);
 	if (blk_chg)
 		write_block (block, blk);
 }
 
 static void
-add_zone_tind2 (unsigned long *znr, int *corrected)
+add_zone_tind2 (unsigned int *znr, int *corrected)
 {
 	static char blk[BLOCK_SIZE];
 	int i, blk_chg = 0;
@@ -868,7 +867,7 @@ add_zone_tind2 (unsigned long *znr, int *corrected)
 		return;
 	read_block (block, blk);
 	for (i = 0; i < BLOCK_SIZE >> 2; i++)
-		add_zone_dind2 (i + (unsigned long *) blk, &blk_chg);
+		add_zone_dind2 (i + (unsigned int *) blk, &blk_chg);
 	if (blk_chg)
 		write_block (block, blk);
 }
@@ -1303,12 +1302,12 @@ int main(int argc, char ** argv)
 		for (i=1,free=0 ; i < INODES ; i++)
 			if (!inode_in_use(i))
 				free++;
-		printf("\n%6d inodes used (%d%%)\n",(INODES-free-1),
+		printf("\n%6ld inodes used (%ld%%)\n",(INODES-free-1),
 			100*(INODES-free-1)/(INODES-1));
 		for (i=FIRSTZONE,free=0 ; i < ZONES ; i++)
 			if (!zone_in_use(i))
 				free++;
-		printf("%6d zones used (%d%%)\n",(ZONES-free),
+		printf("%6ld zones used (%ld%%)\n",(ZONES-free),
 			100*(ZONES-free)/ZONES);
 		printf("\n%6d regular files\n"
 		"%6d directories\n"
