@@ -68,6 +68,7 @@
 #include <stdlib.h>
 #include <termios.h>
 #include <sys/stat.h>
+#include <sys/ioctl.h>
 #include <mntent.h>
 
 #include <linux/fs.h>
@@ -497,11 +498,11 @@ void setup_tables(void)
 	if (!inode_buffer)
 		die("unable to allocate buffer for inodes");
 	memset(inode_buffer,0,INODE_BUFFER_SIZE);
-	printf("%d inodes\n",INODES);
-	printf("%d blocks\n",ZONES);
-	printf("Firstdatazone=%d (%d)\n",FIRSTZONE,NORM_FIRSTZONE);
+	printf("%ld inodes\n",INODES);
+	printf("%ld blocks\n",ZONES);
+	printf("Firstdatazone=%ld (%ld)\n",FIRSTZONE,NORM_FIRSTZONE);
 	printf("Zonesize=%d\n",BLOCK_SIZE<<ZONESIZE);
-	printf("Maxsize=%d\n\n",MAXSIZE);
+	printf("Maxsize=%ld\n\n",MAXSIZE);
 }
 
 /*
@@ -584,7 +585,7 @@ char *filename;
     die("can't open file of bad blocks");
   }
   while(!feof(listfile)) {
-    fscanf(listfile,"%d\n", &blockno);
+    fscanf(listfile,"%ld\n", &blockno);
     mark_zone(blockno);
     badblocks++;
   }
