@@ -113,8 +113,9 @@ extern void closelog();
   * executable small on systems that do not have shared libraries (System V
   * Release <3).
   */
-
+#ifndef BUFSIZ
 #define	BUFSIZ		1024
+#endif
 
  /*
   * When multiple baud rates are specified on the command line, the first one
@@ -173,41 +174,41 @@ struct Speedtab {
 };
 
 static struct Speedtab speedtab[] = {
-    50, B50,
-    75, B75,
-    110, B110,
-    134, B134,
-    150, B150,
-    200, B200,
-    300, B300,
-    600, B600,
-    1200, B1200,
-    1800, B1800,
-    2400, B2400,
-    4800, B4800,
-    9600, B9600,
+    { 50, B50 },
+    { 75, B75 },
+    { 110, B110 },
+    { 134, B134 },
+    { 150, B150 },
+    { 200, B200 },
+    { 300, B300 },
+    { 600, B600 },
+    { 1200, B1200 },
+    { 1800, B1800 },
+    { 2400, B2400 },
+    { 4800, B4800 },
+    { 9600, B9600 },
 #ifdef	B19200
-    19200, B19200,
+    { 19200, B19200 },
 #endif
 #ifdef	B38400
-    38400, B38400,
+    { 38400, B38400 },
 #endif
 #ifdef	EXTA
-    19200, EXTA,
+    { 19200, EXTA },
 #endif
 #ifdef	EXTB
-    38400, EXTB,
+    { 38400, EXTB },
 #endif
 #ifdef B57600
-    57600, B57600,
+    { 57600, B57600 },
 #endif
 #ifdef B115200
-    115200, B115200,
+    { 115200, B115200 },
 #endif
 #ifdef B230400
-    230400, B230400,
+    { 230400, B230400 },
 #endif
-    0, 0,
+    { 0, 0 },
 };
 
 #define P_(s) ()
@@ -503,8 +504,8 @@ void
 update_utmp(line)
      char   *line;
 {
-    struct utmp ut;
-    long    ut_size = sizeof(ut);	/* avoid nonsense */
+    struct  utmp ut;
+    time_t  t;
     int     ut_fd;
     int     mypid = getpid();
     long    time();
@@ -535,27 +536,37 @@ update_utmp(line)
 	memset(&ut, 0, sizeof(ut));
 	strncpy(ut.ut_id, line + 3, sizeof(ut.ut_id));
     }
-    endutent();
+    /*endutent();*/
 	
     strncpy(ut.ut_user, "LOGIN", sizeof(ut.ut_user));
     strncpy(ut.ut_line, line, sizeof(ut.ut_line));
-    time(&ut.ut_time);
+    time(&t);
+    ut.ut_time = t;
     ut.ut_type = LOGIN_PROCESS;
     ut.ut_pid = mypid;
 
     pututline(&ut);
     endutent();
 
-    if((ut_fd = open(_PATH_WTMP, O_APPEND|O_WRONLY)) >= 0) {
-	flock(ut_fd, LOCK_EX);
-	write(ut_fd, &ut, sizeof(ut));
-	flock(ut_fd, LOCK_UN);
-	close(ut_fd);
+    {
+	int lf;
+
+	if ((lf = open(_PATH_WTMPLOCK, O_CREAT|O_WRONLY, 0660)) >= 0) {
+	    flock(lf, LOCK_EX);
+	    if ((ut_fd = open(_PATH_WTMP, O_APPEND|O_WRONLY)) >= 0) {
+		write(ut_fd, &ut, sizeof(ut));
+		close(ut_fd);
+	    }
+	    flock(lf, LOCK_UN);
+	    close(lf);
+	}
     }
 #else
     if ((ut_fd = open(UTMP_FILE, 2)) < 0) {
 	error("%s: open for update: %m", UTMP_FILE);
     } else {
+	long    ut_size = sizeof(ut);       /* avoid nonsense */
+
 	while (read(ut_fd, (char *) &ut, sizeof(ut)) == sizeof(ut)) {
 	    if (ut.ut_type == INIT_PROCESS && ut.ut_pid == mypid) {
 		ut.ut_type = LOGIN_PROCESS;
@@ -805,7 +816,6 @@ do_prompt(op, tp)
 		    break;
 
 		  case 'o':
-#ifdef __alpha__
 		   {
 		     char domainname[256];
 
@@ -813,10 +823,7 @@ do_prompt(op, tp)
 		     domainname[sizeof(domainname)-1] = '\0';
 		     printf ("%s", domainname);
 		   }
-#else
-		    (void) printf ("%s", uts.domainname);
-#endif
-		    break;
+		  break;
 
 		  case 'd':
 		  case 't':
@@ -1205,7 +1212,7 @@ error(va_alist)
      */
 
 #ifdef	USE_SYSLOG
-    (void) openlog(progname, LOG_PID, LOG_AUTH);
+    (void) openlog(progname, LOG_PID, LOG_AUTHPRIV);
     (void) syslog(LOG_ERR, "%s", buf);
     closelog();
 #else
