@@ -3,6 +3,8 @@
    Fixed by JDS June 1996 to clear lists and close files
 */
 
+#define _GNU_SOURCE		/* for snprintf */
+
 #include <sys/types.h>
 #include <sys/param.h>
 
@@ -26,7 +28,7 @@
 #include "pathnames.h"
 
 /* functions in login.c */
-void badlogin(char *s);
+void badlogin(const char *s);
 void sleepexit(int);
 extern struct hostent hostaddress;
 extern char *hostname;
@@ -36,7 +38,7 @@ struct hostent hostaddress;
 char *hostname;
 
 void 
-badlogin(char *s)
+badlogin(const char *s)
 {
     printf("badlogin: %s\n", s);
 }
@@ -88,7 +90,7 @@ am_in_group(char *group)
 }
 
 static void
-find_groups(gid_t defgrp, char *user)
+find_groups(gid_t defgrp, const char *user)
 {
 	num_groups = getgroups(NGROUPS, mygroups);
 }
@@ -108,6 +110,7 @@ new_class(char *class)
     tc->next = ttyclasses;
     tc->first = NULL;
     strncpy(tc->classname, class, CLASSNAMELEN);
+    tc->classname[CLASSNAMELEN-1] = 0;
     ttyclasses = tc;
     return tc;
 }
@@ -128,19 +131,19 @@ add_to_class(struct ttyclass *tc, char *tty)
 
     ge->next = tc->first;
     strncpy(ge->name, tty, NAMELEN);
+    ge->name[NAMELEN-1] = 0;
     tc->first = ge;
 }
 
 
 /* return true if tty is a pty. Very linux dependent */
 static int
-isapty(char *tty)
+isapty(const char *tty)
 {
     char devname[100];
     struct stat stb;
 
-    strcpy(devname, "/dev/");
-    strncat(devname, tty, 80);
+    snprintf(devname, sizeof(devname), "/dev/%s", tty);
 
 #if defined(__linux__) && defined(PTY_SLAVE_MAJOR)
     /* this is for linux 1.3 and newer */
@@ -148,8 +151,10 @@ isapty(char *tty)
        && major(stb.st_rdev) == PTY_SLAVE_MAJOR) {
 	return 1;
     }
-#elif defined(__linux__)
-    /* this is for linux versions before 1.3 */
+#endif
+
+#if defined(__linux__)
+    /* this is for linux versions before 1.3, backward compat. */
     if((stat(devname, &stb) >= 0)
        && major(stb.st_rdev) == TTY_MAJOR
        && minor(stb.st_rdev) >= 192) {
@@ -161,7 +166,7 @@ isapty(char *tty)
 
 /* match the hostname hn against the pattern pat */
 static int
-hnmatch(char *hn, char *pat)
+hnmatch(const char *hn, const char *pat)
 {
     int x1, x2, x3, x4, y1, y2, y3, y4;
     unsigned long p, mask, a;
@@ -238,7 +243,7 @@ timeok(struct tm *t, char *spec)
    Also return true if hostname matches the hostname pattern, class
    or a pattern in the class named by class. */
 static int
-in_class(char *tty, char *class)
+in_class(const char *tty, char *class)
 {
     struct ttyclass *tc;
     struct grplist *ge;
@@ -254,7 +259,8 @@ in_class(char *tty, char *class)
     if (class[0] == '[') {
 	if ((p = strchr(class, ']'))) {
 	    *p = 0;
-	    strcpy(timespec, class+1);
+	    strncpy(timespec, class+1, sizeof(timespec));
+	    timespec[sizeof(timespec)-1] = 0;
 	    *p = ']';
 	    if(!timeok(tm, timespec)) return 0;
 	    class = p+1;
@@ -275,7 +281,8 @@ in_class(char *tty, char *class)
 		if (n[0] == '[') {
 		    if ((p = strchr(n, ']'))) {
 			*p = 0;
-			strcpy(timespec, n+1);
+			strncpy(timespec, n+1, sizeof(timespec));
+			timespec[sizeof(timespec)-1] = 0;
 			*p = ']';
 			if(!timeok(tm, timespec)) continue;
 			n = p+1;
@@ -328,7 +335,7 @@ free_all(void)
 /* end JDS - SBA */
 
 void
-checktty(char *user, char *tty, struct passwd *pwd)
+checktty(const char *user, const char *tty, struct passwd *pwd)
 {
     FILE *f;
     char buf[256], defaultbuf[256];
@@ -359,6 +366,7 @@ checktty(char *user, char *tty, struct passwd *pwd)
 
 	if (buf[0] == '*') {
 	    strncpy(defaultbuf, buf, 256);
+	    defaultbuf[255] = 0;
 	    continue;
 	}
 
