@@ -6,9 +6,9 @@
  *   modify it under the terms of the gnu general public license.
  *   there is no warranty.
  *
- *   $Author: janl $
- *   $Revision: 1.14 $
- *   $Date: 1996/11/11 22:26:21 $
+ *   $Author: aebr $
+ *   $Revision: 1.16 $
+ *   $Date: 1997/07/06 00:12:08 $
  *
  * Updated Thu Oct 12 09:33:15 1995 by faith@cs.unc.edu with security
  *   patches from Zefram <A.Main@dcs.warwick.ac.uk>
@@ -24,8 +24,9 @@
  *
  */
 
-static char rcsId[] = "$Version: $Id: chsh.c,v 1.14 1996/11/11 22:26:21 janl Exp $ $";
+static char rcsId[] = "$Version: $Id: chsh.c,v 1.16 1997/07/06 00:12:08 aebr Exp $ $";
 
+#define _XOPEN_SOURCE		/* to get definition of crypt() */
 #if 0
 #define _POSIX_SOURCE 1
 #endif
@@ -41,6 +42,11 @@ static char rcsId[] = "$Version: $Id: chsh.c,v 1.14 1996/11/11 22:26:21 janl Exp
 #include <getopt.h>
 #include "../version.h"
 
+#if REQUIRE_PASSWORD && USE_PAM
+#include <security/pam_appl.h>
+#include <security/pam_misc.h>
+#endif
+
 extern int is_local(char *);
 
 #undef P
@@ -85,6 +91,11 @@ int main (argc, argv)
     struct sinfo info;
     struct passwd *pw;
     extern int errno;
+#if REQUIRE_PASSWORD && USE_PAM
+    pam_handle_t *pamh = NULL;
+    int retcode;
+    struct pam_conv conv = { misc_conv, NULL };
+#endif
 
     /* whoami is the program name for error messages */
     whoami = argv[0];
@@ -133,16 +144,43 @@ int main (argc, argv)
 
     printf( "Changing shell for %s.\n", pw->pw_name );
 
-#ifdef REQUIRE_PASSWORD
+#if REQUIRE_PASSWORD
+# if USE_PAM
+    if(uid != 0) {
+        if (pam_start("chsh", pw->pw_name, &conv, &pamh)) {
+	    puts("Password error.");
+	    exit(1);
+	}
+        if (pam_authenticate(pamh, 0)) {
+	    puts("Password error.");
+	    exit(1);
+	}
+        retcode = pam_acct_mgmt(pamh, 0);
+        if (retcode == PAM_AUTHTOKEN_REQD) {
+	    retcode = pam_chauthtok(pamh, PAM_CHANGE_EXPIRED_AUTHTOK);
+        } else if (retcode) {
+	    puts("Password error.");
+	    exit(1);
+	}
+        if (pam_setcred(pamh, 0)) {
+	    puts("Password error.");
+	    exit(1);
+	}
+        /* no need to establish a session; this isn't a session-oriented
+         * activity... */
+    }
+# else /* USE_PAM */
     /* require password, unless root */
     if(uid != 0 && pw->pw_passwd && pw->pw_passwd[0]) {
 	pwdstr = getpass("Password: ");
-	if(strncmp(pw->pw_passwd, crypt(pwdstr, pw->pw_passwd), 13)) {
+	if(strncmp(pw->pw_passwd,
+		   crypt(pwdstr, pw->pw_passwd), 13)) {
 	    puts("Incorrect password.");
 	    exit(1);
 	}
     }
-#endif
+# endif /* USE_PAM */
+#endif /* REQUIRE_PASSWORD */
 
     if (! shell) {
 	shell = prompt ("New shell", oldshell);
