@@ -1,9 +1,8 @@
 /* 
- 
- passwd.c - change password on an account
- 
- Initially written for Linux by Peter Orbaek <poe@daimi.aau.dk>
- Currently maintained at ftp://ftp.daimi.aau.dk/pub/linux/poe/
+ * passwd.c - change password on an account
+ *
+ * Initially written for Linux by Peter Orbaek <poe@daimi.aau.dk>
+ * Currently maintained at ftp://ftp.daimi.aau.dk/pub/linux/poe/
 
  Hacked by Alvaro Martinez Echevarria, alvaro@enano.etsit.upm.es,
  to allow peaceful coexistence with yp. Nov 94.
@@ -15,11 +14,8 @@
  moved Alvaro's changes to setpwnam.c (so they get used
  by chsh and chfn as well). Oct 5, 96.
 
- Someone broke Alvaro's yp patch.  Now it lives in setpwnam.
-
  */
 
-
 /*
  * Sun Oct 15 13:18:34 1995  Martin Schulze  <joey@finlandia.infodrom.north.de>
  *
@@ -52,6 +48,7 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <stdarg.h>
 #include <termios.h>
 #include <getopt.h>
 #include <malloc.h>
@@ -62,7 +59,16 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/resource.h>
-#include "../version.h"
+
+#if defined (__GNU_LIBRARY__) && __GNU_LIBRARY__ > 1
+#include <crypt.h>
+#endif
+
+#if 0
+#  include "../version.h"
+#else
+char version[] = "admutil 1.18, 15-Oct-95";
+#endif
 
 #ifndef _PATH_CHFN
 # define _PATH_CHFN "/usr/bin/chfn"
@@ -82,10 +88,15 @@ extern int setpwnam(struct passwd *);	/* setpwnam.c */
 #define bin_to_ascii(c) ((c)>=38?((c)-38+'a'):(c)>=12?((c)-12+'A'):(c)+'.')
 
 static void
-pexit(str)
-     char *str;
+pexit(char *str, ...)
 {
-    perror(str);
+    va_list vlst;
+
+    va_start(vlst, str);
+    vfprintf(stderr, str, vlst);
+    fprintf(stderr, ": ");
+    perror("");
+    va_end(vlst);
     exit(1);
 }
 
@@ -182,6 +193,7 @@ int check_passwd(char *passwd, char *oldpasswd, char *user, char *gecos)
     if ( (c = index(gecos, ',')) ) {
 	if ( c-gecos && (g = (char *)malloc (c-gecos+1)) ) {
 	    strncpy (g, gecos, c-gecos);
+	    g[c-gecos] = 0;
 	    while ( (c=rindex(g, ' ')) ) {
 		if ( !check_passwd_string(passwd, c+1) ) {
 		    printf("Please don't use something like your realname as password!\n");
@@ -257,7 +269,7 @@ main(argc, argv)
 	    break;
 	case 'V':
 	case 'v':
-	    printf("%s\n", util_linux_version);
+	    printf("%s\n", version);
 	    exit(0);
 	default:
 	    fprintf(stderr, "Usage: passwd [-foqsvV] [user [password]]\n");
@@ -359,6 +371,7 @@ main(argc, argv)
 	    goto redo_it;
 	
 	strncpy(pwdstr1, pwdstr, 9);
+	pwdstr1[9] = 0;
 	pwdstr = getpass("Re-type new password: ");
 	
 	if(strncmp(pwdstr, pwdstr1, 8)) {
@@ -367,7 +380,7 @@ main(argc, argv)
 	}
     } /* pwdstr i.e. password set on command line */
     
-    time(&tm);
+    time(&tm); tm ^= getpid();
     salt[0] = bin_to_ascii(tm & 0x3f);
     salt[1] = bin_to_ascii((tm >> 6) & 0x3f);
     cryptstr = crypt(pwdstr, salt);
