@@ -176,7 +176,7 @@ main(argc, argv)
 	argc -= optind;
 	argv += optind;
 
-	month = 0;
+	month = year = 0;
 	switch(argc) {
 	case 2:
 		if ((month = atoi(*argv++)) < 1 || month > 12)
@@ -221,7 +221,7 @@ void headers_init(void)
   strcpy(j_day_headings,"");
   
   for(i = 0 ; i < 7 ; i++ ) {
-#if defined(__linux__) && _LINUX_C_LIB_VERSION_MAJOR > 4
+#if defined(__linux__) && (_LINUX_C_LIB_VERSION_MAJOR > 4 || __GNU_LIBRARY__ > 1)
      strncat(day_headings,nl_langinfo(ABDAY_1+i),2);
      strcat(j_day_headings,nl_langinfo(ABDAY_1+i));
 #else
@@ -233,7 +233,7 @@ void headers_init(void)
   }
   
   for (i = 0; i < 12; i++) {
-#if defined(__linux__) && _LINUX_C_LIB_VERSION_MAJOR > 4
+#if defined(__linux__) && (_LINUX_C_LIB_VERSION_MAJOR > 4 || __GNU_LIBRARY__ > 1)
      full_month[i] = nl_langinfo(MON_1+i);
 #else
      full_month[i] = _time_info->full_month[i];
