@@ -1,5 +1,7 @@
 #include <stdio.h>
+#include <unistd.h>
 
+int
 main(argc, argv)
 int argc;
 char *argv[];
@@ -26,5 +28,5 @@ char *argv[];
 	for (; optind < argc; optind++)
 		printf(" %s", argv[optind]);
 	printf("\n");
-	exit(status);
+	return status;
 }
