@@ -42,10 +42,12 @@ chdir to /,  or if it encounters an unknown file type.
 -------------------------------------------------------------*/
 
 #ifndef lint
-static char *RCSid = "$Id: namei.c,v 1.5 1996/07/02 16:58:01 janl Exp $";
+static char *RCSid = "$Id: namei.c,v 1.6 1997/07/06 00:13:09 aebr Exp $";
 #endif
 
 #include <stdio.h>
+#include <unistd.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
@@ -67,6 +69,7 @@ int xflag = 0;
 
 static char *pperm();
 
+int
 main(argc, argv)
 int argc;
 char *argv[];
@@ -113,7 +116,7 @@ char *argv[];
 	    exit(1);
 	}
     }
-    exit(0);
+    return 0;
 }
 
 void
