@@ -47,6 +47,17 @@ static char sccsid[] = "@(#)whereis.c	5.5 (Berkeley) 4/18/91";
 #include <string.h>
 #include <ctype.h>
 
+void zerof(void);
+void getlist(int *, char ***, char ***, int *);
+void lookup(char *);
+void looksrc(char *);
+void lookbin(char *);
+void lookman(char *);
+void findv(char **, int, char *);
+void find(char **, char *);
+void findin(char *, char *);
+int itsit(char *, char *);
+
 static char *bindirs[] = {
 #ifdef __linux__
    "/bin",
@@ -234,6 +245,7 @@ char	uflag;
  * whereis name
  * look for source, documentation and binaries
  */
+int
 main(argc, argv)
 	int argc;
 	char *argv[];
@@ -291,9 +303,10 @@ usage:
 		} else
 			lookup(*argv++);
 	while (--argc > 0);
-	exit(0);
+	return 0;
 }
 
+void
 getlist(argcp, argvp, flagp, cntp)
 	char ***argvp;
 	int *argcp;
@@ -311,16 +324,17 @@ getlist(argcp, argvp, flagp, cntp)
 }
 
 
+void
 zerof()
 {
-
 	if (sflag && bflag && mflag)
 		sflag = bflag = mflag = 0;
 }
+
 int	count;
 int	print;
 
-
+void
 lookup(cp)
 	register char *cp;
 {
@@ -372,6 +386,7 @@ again:
 		printf("\n");
 }
 
+void
 looksrc(cp)
 	char *cp;
 {
@@ -381,6 +396,7 @@ looksrc(cp)
 		findv(Sflag, Scnt, cp);
 }
 
+void
 lookbin(cp)
 	char *cp;
 {
@@ -390,6 +406,7 @@ lookbin(cp)
 		findv(Bflag, Bcnt, cp);
 }
 
+void
 lookman(cp)
 	char *cp;
 {
@@ -399,6 +416,7 @@ lookman(cp)
 		findv(Mflag, Mcnt, cp);
 }
 
+void
 findv(dirv, dirc, cp)
 	char **dirv;
 	int dirc;
@@ -409,6 +427,7 @@ findv(dirv, dirc, cp)
 		findin(*dirv++, cp), dirc--;
 }
 
+void
 find(dirs, cp)
 	char **dirs;
 	char *cp;
@@ -418,6 +437,7 @@ find(dirs, cp)
 		findin(*dirs++, cp);
 }
 
+void
 findin(dir, cp)
 	char *dir, *cp;
 {
@@ -437,6 +457,7 @@ findin(dir, cp)
 	closedir(dirp);
 }
 
+int
 itsit(cp, dp)
 	register char *cp, *dp;
 {
