@@ -52,16 +52,19 @@ static char copyright[] =
 static char sccsid[] = "@(#)write.c	8.1 (Berkeley) 6/6/93";
 #endif /* not lint */
 
-#include <sys/param.h>
-#include <sys/signal.h>
-#include <sys/stat.h>
-#include <sys/file.h>
-#include <sys/time.h>
+#define _GNU_SOURCE		/* for snprintf */
+
+#include <unistd.h>
 #include <utmp.h>
 #include <ctype.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/param.h>
+#include <sys/signal.h>
+#include <sys/stat.h>
+#include <sys/file.h>
+#include <sys/time.h>
 #ifdef __linux__
 #include <paths.h>
 #include "pathnames.h"
@@ -71,10 +74,13 @@ static char sccsid[] = "@(#)write.c	8.1 (Berkeley) 6/6/93";
 void search_utmp(char *, char *, char *, uid_t);
 void do_write(char *, char *, uid_t);
 void wr_fputs(char *);
+int term_chk(char *, int *, time_t *, int);
+int utmp_chk(char *, char *);
 
 extern int errno;
 
-int main(int argc, char **argv)
+void
+main(int argc, char **argv)
 {
 	register char *cp;
 	time_t atime;
@@ -102,7 +108,7 @@ int main(int argc, char **argv)
 		(void)fprintf(stderr, "write: can't find your tty's name\n");
 		exit(1);
 	  }
-	  if (cp = rindex(mytty, '/'))
+	  if ((cp = rindex(mytty, '/')) != NULL)
 		mytty = cp + 1;
 	  if (term_chk(mytty, &msgsok, &atime, 1))
 		exit(1);
@@ -160,20 +166,23 @@ int utmp_chk(char *user, char *tty)
 
 {
 	struct utmp u;
-	int ufd;
+	struct utmp *uptr;
+	int res = 1;
 
-	if ((ufd = open(_PATH_UTMP, O_RDONLY)) < 0)
-		return(0);	/* ignore error, shouldn't happen anyway */
+	utmpname(_PATH_UTMP);
+	setutent();
 
-	while (read(ufd, (char *) &u, sizeof(u)) == sizeof(u))
+	while ((uptr = getutent())) {
+		memcpy(&u, uptr, sizeof(u));
 		if (strncmp(user, u.ut_name, sizeof(u.ut_name)) == 0 &&
 		    strncmp(tty, u.ut_line, sizeof(u.ut_line)) == 0) {
-			(void)close(ufd);
-			return(0);
+			res = 0;
+			break;
 		}
+	}
 
-	(void)close(ufd);
-	return(1);
+	endutent();
+	return(res);
 }
 
 /*
@@ -191,23 +200,23 @@ void search_utmp(char *user, char *tty, char *mytty, uid_t myuid)
 
 {
 	struct utmp u;
+	struct utmp *uptr;
 	time_t bestatime, atime;
-	int ufd, nloggedttys, nttys, msgsok, user_is_me;
+	int nloggedttys, nttys, msgsok, user_is_me;
 #ifdef __linux__
 	char atty[sizeof(u.ut_line) + 1];
 #else
 	char atty[UT_LINESIZE + 1];
 #endif
 
-	if ((ufd = open(_PATH_UTMP, O_RDONLY)) < 0) {
-		perror("utmp");
-		exit(1);
-	}
+	utmpname(_PATH_UTMP);
+	setutent();
 
 	nloggedttys = nttys = 0;
 	bestatime = 0;
 	user_is_me = 0;
-	while (read(ufd, (char *) &u, sizeof(u)) == sizeof(u))
+	while ((uptr = getutent())) {
+		memcpy(&u, uptr, sizeof(u));
 		if (strncmp(user, u.ut_name, sizeof(u.ut_name)) == 0) {
 			++nloggedttys;
 #ifdef __linux__
@@ -235,8 +244,9 @@ void search_utmp(char *user, char *tty, char *mytty, uid_t myuid)
 				(void)strcpy(tty, atty);
 			}
 		}
+	}
 
-	(void)close(ufd);
+	endutent();
 	if (nloggedttys == 0) {
 		(void)fprintf(stderr, "write: %s is not logged in\n", user);
 		exit(1);
@@ -266,7 +276,7 @@ int term_chk(char *tty, int *msgsokP, time_t *atimeP, int showerror)
 	struct stat s;
 	char path[MAXPATHLEN];
 
-	(void)sprintf(path, "/dev/%s", tty);
+	(void)snprintf(path, sizeof(path), "/dev/%s", tty);
 	if (stat(path, &s) < 0) {
 		if (showerror)
 			(void)fprintf(stderr,
@@ -292,12 +302,12 @@ void do_write(char *tty, char *mytty, uid_t myuid)
 
 	/* Determine our login name before the we reopen() stdout */
 	if ((login = getlogin()) == NULL)
-		if (pwd = getpwuid(myuid))
+		if ((pwd = getpwuid(myuid)) != NULL)
 			login = pwd->pw_name;
 		else
 			login = "???";
 
-	(void)sprintf(path, "/dev/%s", tty);
+	(void)snprintf(path, sizeof(path), "/dev/%s", tty);
 	if ((freopen(path, "w", stdout)) == NULL) {
 		(void)fprintf(stderr, "write: %s: %s\n", path, strerror(errno));
 		exit(1);
