@@ -9,16 +9,24 @@
 #include <linux/unistd.h>
 #include <stdio.h>
 #include <getopt.h>
+#include <stdlib.h>
 
 #if __GNU_LIBRARY__ < 5
+
+#ifndef __alpha__
 # define __NR_klogctl __NR_syslog
   static inline _syscall3(int, klogctl, int, type, char *, b, int, len);
+#else /* __alpha__ */
+#define klogctl syslog
+#endif
+
 #else
 # include <sys/klog.h>
 #endif
 
 static char *progname;
 
+void
 usage()
 {
    fprintf( stderr, "Usage: %s [-c] [-n level]\n", progname );
@@ -30,7 +38,7 @@ int main( int argc, char *argv[] )
    int  i;
    int  n;
    int  c;
-   int  level;
+   int  level = 0;
    int  lastc;
    int  cmd = 3;
 
