@@ -26,13 +26,10 @@
   
   The program is designed to run setuid superuser, since we need to be
   able to do direct I/O.  (More to the point: we need permission to 
-  execute the iopl() system call).  (However, if you use one of the 
+  execute the iopl() system call.)  (However, if you use one of the 
   methods other than direct ISA I/O to access the clock, no setuid is
   required).
  
-  I don't know what the Hardware Clock will do in 2000, so this program
-  probably won't work past the century boundary.
-
   Here's some info on how we must deal with the time that elapses while
   this program runs: There are two major delays as we run:
 
@@ -78,15 +75,12 @@
 
    - When waiting for whole second boundary in set_hardware_clock_exact,
      fail if we miss the goal by more than .1 second, as could happen if
-     we get pre-empted.
-
-  Unauthorized hacking by Nicolai Langfeldt (janl@math.uio.no) to make
-  it compile on linux 1.2 machines as well as more recent versions of
-  the kernel. Introduced the NO_CLOCK access method and wrote feature
-  test code to detect absense of rtc headers.
+     we get pre-empted (by the kernel dispatcher).
 
 ****************************************************************************/ 
 
+#define _GNU_SOURCE             /* for snprintf */
+
 #include <string.h>
 #include <stdio.h>
 #include <fcntl.h>
@@ -102,7 +96,7 @@
 #include "../version.h"
 
 #define MYNAME "hwclock"
-#define VERSION "2.0.1"
+#define VERSION "2.1"
 
 #define FLOOR(arg) ((arg >= 0 ? (int) arg : ((int) arg) - 1));
 
@@ -135,7 +129,7 @@ struct adjtime {
 };
 
 
-enum clock_access_method {ISA, RTC_IOCTL, KD, NO_CLOCK};
+enum clock_access_method {ISA, RTC_IOCTL, KD};
   /* A method for accessing (reading, writing) the hardware clock:
      
      ISA: 
@@ -241,7 +235,10 @@ hclock_read(unsigned char reg) {
 #ifdef __i386__
   register unsigned char ret;
   __asm__ volatile ("cli");
-  outb(reg | 0x80, clock_ctl_addr);
+  /* & 0x7f ensures that we are not disabling NMI while we read.
+     Setting on Bit 7 here would disable NMI
+     */
+  outb(reg & 0x7f, clock_ctl_addr);
   ret = inb(clock_data_addr);
   __asm__ volatile ("sti");
   return ret;
@@ -256,7 +253,10 @@ hclock_write(unsigned char reg, unsigned char val) {
   Set relative byte <reg> of the Hardware Clock value to <val>.
 ----------------------------------------------------------------------------*/
 #ifdef __i386__
-  outb(reg | 0x80, clock_ctl_addr);
+  /* & 0x7f ensures that we are not disabling NMI while we read.
+     Setting on Bit 7 here would disable NMI
+     */
+  outb(reg & 0x7f, clock_ctl_addr);
   outb(val, clock_data_addr);
 #endif
 }
@@ -504,11 +504,10 @@ mktime_tz(struct tm tm, const bool universal) {
   }
   
   /* now put back the original zone.  */
-  if (zone) {
-    char zonebuf[strlen(zone) + 4];
-    sprintf(zonebuf, "TZ=%s", zone);
-    putenv(zonebuf);
-  } else putenv ("TZ");
+  if (zone)
+    setenv ("TZ", zone, 1);
+  else
+    putenv ("TZ");
   tzset();
 
   if (debug) 
@@ -619,10 +618,26 @@ read_hardware_clock_isa(struct tm *tm) {
     tm->tm_mday = hclock_read_bcd(7);
     tm->tm_mon = hclock_read_bcd(8);
     tm->tm_year = hclock_read_bcd(9);
+    if (hclock_read_bcd(50) == 0) {
+      /* I suppose Linux could run on an old machine that doesn't implement
+         the Byte 50 century value, and that if it does, that machine puts
+         zero in Byte 50.  If so, this could could be useful, in that it
+         makes values 70-99 -> 1970-1999 and 00-69 -> 2000-2069.
+         */
+      if (hclock_read_bcd(9) >= 70) tm->tm_year = hclock_read_bcd(9);
+      else tm->tm_year = hclock_read_bcd(9) + 100;
+    } else {
+      tm->tm_year = hclock_read_bcd(50) * 100 + hclock_read_bcd(9) - 1900;
+      /* Note: Byte 50 contains centuries since A.D.  Byte 9 contains
+         years since beginning of century.  tm_year contains years
+         since 1900.  At least we _assume_ that's what tm_year
+         contains.  It is documented only as "year", and it could
+         conceivably be years since the beginning of the current
+         century.  If so, this code won't work after 1999.  
+         */
+    }
   } while (tm->tm_sec != hclock_read_bcd (0));
 
-  if (tm->tm_year < 70) tm->tm_year += 100;  
-    /* 70..99 => 1970..1999, 0..69 => 2000..2069 */
   tm->tm_mon--;	           /* DOS uses 1 base */
   tm->tm_wday -= 3;         /* DOS uses 3 - 9 for week days */
   tm->tm_isdst = -1;        /* don't know whether it's daylight */
@@ -752,7 +767,9 @@ set_hardware_clock_isa(const struct tm new_broken_time,
   if (testing) 
     printf("Not setting Hardware Clock because running in test mode.\n");
   else {
+#ifdef __i386__
     __asm__ volatile ("cli");
+#endif
     save_control = hclock_read(11);   /* tell the clock it's being set */
     hclock_write(11, (save_control | 0x80));
     save_freq_select = hclock_read(10);       /* stop and reset prescaler */
@@ -764,7 +781,8 @@ set_hardware_clock_isa(const struct tm new_broken_time,
     hclock_write_bcd(6, new_broken_time.tm_wday + 3);
     hclock_write_bcd(7, new_broken_time.tm_mday);
     hclock_write_bcd(8, new_broken_time.tm_mon + 1);
-    hclock_write_bcd(9, new_broken_time.tm_year);
+    hclock_write_bcd(9, new_broken_time.tm_year%100);
+    hclock_write_bcd(50, (1900+new_broken_time.tm_year)/100);
     
     /* The kernel sources, linux/arch/i386/kernel/time.c, have the
        following comment:
@@ -782,7 +800,9 @@ set_hardware_clock_isa(const struct tm new_broken_time,
 
     hclock_write (11, save_control);
     hclock_write (10, save_freq_select);
+#ifdef __i386__
     __asm__ volatile ("sti");
+#endif
   }
 }
 
@@ -1226,10 +1246,26 @@ determine_clock_access_method(const bool user_requests_ISA,
 
   <user_requests_ISA> means the user explicitly asked for the ISA method.
 -----------------------------------------------------------------------------*/
+  bool rtc_works;
+    /* The /dev/rtc method is available and seems to work on this machine */
 
-  *clock_access_p=NO_CLOCK;
+  if (got_rtc) {
+    int rtc_fd = open("/dev/rtc", O_RDONLY);
+    if (rtc_fd > 0) {
+      rtc_works = TRUE;
+      close(rtc_fd);
+    } else {
+      rtc_works = FALSE;
+      if (debug)
+        printf("Open of /dev/rtc failed, errno = %s (%d).  "
+               "falling back to more primitive clock access method.\n",
+               strerror(errno), errno);
+    }
+  } else rtc_works = TRUE;
 
-  if (got_kdghwclk) {
+  if (user_requests_ISA) *clock_access_p = ISA;
+  else if (rtc_works) *clock_access_p = RTC_IOCTL;
+  else if (got_kdghwclk) {
     int con_fd;
     struct hwclk_time t;
 
@@ -1256,22 +1292,7 @@ determine_clock_access_method(const bool user_requests_ISA,
     }
     close(con_fd);
   } else {
-    int rtc_fd;
-    
-    if (user_requests_ISA) *clock_access_p = ISA;
-    else {
-      rtc_fd = (got_rtc)?(open("/dev/rtc", O_RDONLY)):(-1);
-      if (rtc_fd < 0) {
-        *clock_access_p = ISA;
-        if (debug)
-          printf("Open of /dev/rtc failed, errno = %s (%d).  "
-                 "falling back to direct I/O instruction access.\n",
-                 strerror(errno), errno);
-      } else {
-        *clock_access_p = RTC_IOCTL;
-        close(rtc_fd);
-      }
-    }
+    *clock_access_p = ISA;
   }
   if (debug) {
     switch (*clock_access_p) {
@@ -1283,10 +1304,6 @@ determine_clock_access_method(const bool user_requests_ISA,
              *clock_access_p);
     }
   }
-  if (*clock_access_p == NO_CLOCK) {
-    fprintf(stderr,
-	    "Unable to determine clock access scheme for this machine\n");
-  }
 }
 
 
@@ -1501,6 +1518,20 @@ main(int argc, char **argv, char **envp) {
 
   History of this program:
 
+  97.06.01: BJH.  Version 2.1.  Read and write the century byte (Byte
+  50) of the ISA Hardware Clock when using direct ISA I/O.  Problem
+  discovered by job (jei@iclnl.icl.nl).
+  
+  Use the rtc clock access method in preference to the KDGHWCLK method.
+  Problem discovered by Andreas Schwab <schwab@LS5.informatik.uni-dortmund.de>.
+
+  November 1996: Version 2.0.1.  Modifications by Nicolai Langfeldt
+  (janl@math.uio.no) to make it compile on linux 1.2 machines as well
+  as more recent versions of the kernel. Introduced the NO_CLOCK
+  access method and wrote feature test code to detect absense of rtc
+  headers.
+
+
   Bryan Henderson based hwclock on the program "clock", in September
   1996.  While remaining mostly backward compatible with clock,
   hwclock added the following:
@@ -1588,3 +1619,5 @@ main(int argc, char **argv, char **envp) {
   V1.4: alan@SPRI.Levels.UniSA.Edu.Au (Alan Modra)
         Wed Feb  8 12:29:08 1995, fix for years > 2000.
         faith@cs.unc.edu added -v option to print version.  */
+
+
