@@ -60,6 +60,7 @@ beats rebuilding the kernel!
 */
 
 #include <stdio.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <sys/file.h>
 #include <sys/ioctl.h>
@@ -86,7 +87,7 @@ static int ioctl_possible = 0;
 struct kbd_repeat kbdrep_s;
 #endif
 
-void main( int argc, char **argv )
+int main( int argc, char **argv )
 {
    double      rate = 10.9;     /* Default rate */
    int         delay = 250;     /* Default delay */
@@ -195,4 +196,5 @@ void main( int argc, char **argv )
    }
 #endif
 
+   return 0;
 }
