@@ -1,4 +1,4 @@
-/* $Id: shhopt.h,v 2.1 1996/11/11 13:03:52 janl Exp $ */
+/* $Id: shhopt.h,v 2.2 1997/07/06 23:11:58 aebr Exp $ */
 #ifndef SHHOPT_H
 #define SHHOPT_H
 
@@ -28,6 +28,6 @@ typedef struct {
 
 void optSetFatalFunc(void (*f)(const char *, ...));
 void optParseOptions(int *argc, char *argv[],
-		     optStruct opt[], int allowNegNum);
+		     const optStruct opt[], int allowNegNum);
 
 #endif
